// @ts-check

/**
 * Set server port
 */
function setupServerPort(callback) {
  const { isProduction } = require("../config");
  const getPort = require("get-port");
  const log = require("@acanto/workflow-utils/log");
  const chalk = require("@acanto/workflow-utils/chalk");

  if (isProduction) {
    callback();
    return;
  }

  return new Promise((resolve, reject) => {
    getPort({ port: [8080, 3000, 3002, 3004, 3006, 3008] })
      .then((port) => {
        // @ts-ignore
        global.__FREEPORT = port;
        log.info(
          chalk.yellow("Local server will use free port " + chalk.bold(port))
        );
        resolve();
      })
      .catch(reject);
  });
}

function setupMachineIP(callback) {
  const publicIp = require("public-ip");
  const log = require("@acanto/workflow-utils/log");
  const chalk = require("@acanto/workflow-utils/chalk");
  if (!require("ci-info").isCI) {
    return callback();
  }

  return new Promise((resolve, reject) => {
    publicIp
      .v4()
      .then((ip) => {
        // @ts-ignore
        global.__IP = ip;
        log.info(chalk.yellow("Local machine public IP " + chalk.bold(ip)));
        resolve();
      })
      .catch(() => {
        log.info(chalk.yellow("Could not determine local machine public IP."));
        reject();
      });
  });
}

/**
 * Collect subtasks in a single task that respect the gulp async lifecycle by
 * calling the callback on completion.
 */
module.exports = function setup(callback) {
  const gulp = require("gulp");
  return gulp.parallel(setupServerPort, setupMachineIP)(callback);
};
